/**********************************************************************
 * Class OaiTask
 *  
 * Copyright (c) 2008-2012, German National Library / Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany 
 *
 * This program is free software.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Kadir Karaca Kocer -- German National Library
 * 
 **********************************************************************/

package de.dnb.oai.harvester.task;

import java.util.Date;
import java.util.List;

import de.dnb.oai.harvester.Constants;
import de.dnb.oai.repository.Repository;
import de.dnb.stm.manager.ManagerConstants;
import de.dnb.stm.task.Task;
import de.dnb.stm.task.TaskConstants;

/** ********************************************************************
 * Class representing a defined OaiTask.
 * It is mainly a Repository, a waiting Request and a list of finished Requests.
 *
 * @author Kadir Karaca Kocer, German National Library
 * @version 20090220
 * @see de.dnb.oai.harvester.Harvester
 * @see de.dnb.oai.harvester.task.Request
 * @see de.dnb.oai.repository.Repository
 * @since  20080715
 **********************************************************************/

/* ********************************************************************
 * CHANGELOG:
 *   
 * Created on 15.07.2008 by Kadir Karaca Kocer, German National Library
 *********************************************************************/
public class OaiTask implements Task
{
    private long taskId = 0;
    private Repository repository = new Repository(); //repository to harvest 
    private Request request = new Request(); // the waiting request
    private List<Request> processedRequests; // already finished requests -if any
    
	private String verb = Constants.DEFAULT_OAI_VERB;	//which OAI-Action will i perform
    private String identifier = "";		//identifier string
    private String set = "";			//which set to harvest (see OAI spec)
    private String metadataPrefix = "";	//which format must the answer have
    
    private int exportHandlerId = TaskConstants.STATUS_NOT_DEFINED;//
    private Date startDate;
    private int runInterval = 0;	// Run this task again every runInterval seconds. 0-> only once
    //all booleans set by web browser MUST be false by default! (X)HTML tag "checkbox" does not send "false"!
    private boolean validateOaiResponse = false;  //
    private boolean validateRecords = false;  //
    private boolean saveResponse = false;  //

	@SuppressWarnings("unused")
    private Date lastModified;
     
    /**
	 * @return Returns the processedRequests.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public List<Request> getProcessedRequests() {
		return this.processedRequests;
	}

	/**
	 * @param oldRequests The processedRequests to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setProcessedRequests(List<Request> oldRequests) {
		this.processedRequests = oldRequests;
	}
	
    /**
     * Get the task id.
     * 
     * @return The task id
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public long getTaskId() {
        return this.taskId;
    }

    /**
     * Set the task id.
     * 
     * @param newID The new task id
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public void setTaskId(long newID) {
        this.taskId = newID;
    }

    /**
     * Get the task start date.
     * 
     * @return the task start date
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public Date getStartDate() {
        return this.startDate;
    }

    /**
     * Set the task start date.
     * 
     * @param sd The timestamp of first time the task was started
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public void setStartDate(Date sd) {
            this.startDate = sd;
    }
    
    /**
	 * @return The repository
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public Repository getRepository() {
		return this.repository;
	}

	/**
	 * @param repository The repository to set
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setRepository(Repository repository) {
		this.repository = repository;
	}

	/**
     * Get the task "from date".
     * 
     * @return the task start date
     * @author Kadir Karaca Kocer, German National Library
     */
    public Date getFromDate() {
        return this.request.getFromDate();
    }

	/**
	 * @param fd From Date for Harvester
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setFromDate(Date fd) {
		this.request.setFromDate(fd);
	}

	  
    /**
     * Get the task "until date".
     * 
     * @return the task start date
     * @author Kadir Karaca Kocer, German National Library
     */
    public Date getUntilDate() {
        return this.request.getUntilDate();
    }

	/**
	 * @param ud until Date for Harvester
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setUntilDate(Date ud) {
		this.request.setUntilDate(ud);
	}
	

    /**
     * Get the task run interval. If set <= 0 the task will only run once.
     * 
     * @return The task run interval in ms
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public int getRunInterval() {
        return this.runInterval;
    }

    /**
     * Set the task run interval. Deleted by neubauer 29.03.2010: If set < 100 the task will only run once.
     * 
     * @param interval The task run interval in seconds
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public void setRunInterval(int interval) {
            this.runInterval = interval;
    }

	/**
	 * @return Returns the waiting request.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public Request getRequest() {
		return this.request;
	}

	/**
	 * @param newRequest The Request to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setRequest(Request newRequest) {
		this.request = newRequest;
	}

	/**
	 * @return OAI-Identifier of the actual request
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public String getIdentifier() {
		return this.identifier;
	}
	
	/**
	 * @param newIdentifier OAI-Identifier of the actual request
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setIdentifier(String newIdentifier) {
		this.identifier = newIdentifier.trim();
	}
	
	/**
	 * Returns the OAI-PMH metadata set.
	 * 
	 * @return Returns OAI-PMH Set of the request
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public String getOai_Set() {
		return this.set;
	}
	
	/**
	 * Sets OAI-PMH metadata set.
	 * 
	 * @param newSet OAI-PMH Set
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setOai_Set(String newSet) {
		this.set = newSet.trim();
	}
	
	/**
	 * Sets the OAI-PMH metadata prefix.
	 * 
	 * @param prefix OAI-PMH metadata prefix
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setMetadataPrefix(String prefix) {
		this.metadataPrefix = prefix.trim();
	}
	
	/**
	 * Returns the status of actual request.
	 * 
	 * @return The status of actual request
	 * @author Kadir Karaca Kocer, German National Library
	 */
	@Override
    public int getStatus() {
		return this.request.getStatus();
	}

	/**
	 * Sets the status of actual request.
	 * 
	 * @param newStatus New status of actual request
	 * @author Kadir Karaca Kocer, German National Library
	 */
	@Override
    public synchronized void setStatus(int newStatus) {
		this.request.setStatus(newStatus);
	}
	
	/**
	 * @param index Index of the old request in the list of already processed requests.
	 * @return Status code of the request. Error if index is greater than array bounds.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public int getStatusProcessedRequest(int index) {
		if (index < this.processedRequests.size()) {
			return this.processedRequests.get(index).getStatus();
		}
		return ManagerConstants.INDEX_OUT_OF_BOUNDS;
	}
	
	/**
	 * @return Returns the verb.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public String getVerb() {
		return this.verb;
	}
	
	/**
	 * @param newVerb The verb to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setVerb(String newVerb) {
		this.verb = newVerb.trim();
	}
    
    /**
     * @return Returns the exportHandlerId.
     * @author Kadir Karaca Kocer, German National Library
     */
    public int getExportHandlerId() {
        return this.exportHandlerId;
    }
    
    /**
     * @param id The exportHandlerId to set.
     * @author Kadir Karaca Kocer, German National Library
     */
    @Override
    public void setExportHandlerId(int id) {
        this.exportHandlerId = id;
    }

	/**
	 * @return Returns the metadataPrefix.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public String getMetadataPrefix() {
		return this.metadataPrefix;
	}
	
	/**
	 * @return Returns the validateOaiResponse.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public boolean isValidateOaiResponse() {
		return this.validateOaiResponse;
	}

	/**
	 * @param check The validateOaiResponse to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setValidateOaiResponse(boolean check) {
		this.validateOaiResponse = check;
	}

	/**
	 * @return Returns the validateRecords.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public boolean isValidateRecords() {
		return this.validateRecords;
	}

	/**
	 * @param validate The validateRecords to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setValidateRecords(boolean validate) {
		this.validateRecords = validate;
	}

	/**
	 * @return Returns the save_response.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public boolean isSaveResponse() {
		return this.saveResponse;
	}

	/**
	 * @param save The save_response to set.
	 * @author Kadir Karaca Kocer, German National Library
	 */
	public void setSaveResponse(boolean save) {
		this.saveResponse = save;
	}

    /**
     * Returns the Request with the given id.
     * 
     * @param id Id of the request.
     * @return The Request with the given id.
     * @author Kadir Karaca Kocer, German National Library
     */
    public Request getRequestById(long id) {
        if (id == this.request.getRequestId()) {
            return this.request;
        }
        for (int i = 0; i < this.processedRequests.size(); i++) {
            Request r = this.processedRequests.get(i);
            if (id == r.getRequestId()) {
               //found! Return.
               return r;
            }
        }
        return null;
    }
}
